

var AccountModel = $.extend({}, KBObservable.clone(), {
	session: null,
	
	restore: function()
	{
		var session = localStorage.getItem("session");
		
		if(session){
			this.session = JSON.parse(session);
			
			if(this.session.userName){
				TinyCC.options.userName = this.session.userName;
				TinyCC.options.apiKey = this.session.apiKey;
				TinyCC.options.custom_domain = this.session.custom_domain;
			}else{
				window.location.hash="#login";
			}
		}else{
			this.session = null;
			window.location.hash="#login";
		}
		
		this.notifyObservers("loginStateChanged",this.session);
	},
	
	store: function()
	{
		localStorage.setItem("session",JSON.stringify(this.session));
	},
	
	login: function(userName, pass)
	{
		var self = this;
		TinyCC.getApiKey({
			"username": userName,
			"password": btoa(pass), // base64 encoding for minimal security
			"successCallback": function(response){
				//KBDebug.dump(response.results);
				//console.log(response);
				if(!self.session)self.session={};
				self.session.userName = userName;
				self.session.apiKey = response.results.api_key;
				self.session.custom_domain = response.results.custom_domain;
				self.store();
				self.restore();
				window.location.hash="#home";
			},
			"errorCallback": function(message){
				MessageBox.error(message);
			}
		});		
	},
	
	logout: function()
	{
		this.session.userName = null;
		this.session.apiKey = null;
		this.store();
		this.restore();
		UrlsList.firstTimeReloadDone = false;
	}
});

var RecentList = $.extend({}, KBObservable.clone(), {
	"list":[],
	
	"update": function(hash, urlInfo)
	{
		for(var i=0;i<this.list.length;i++)
		{
			if(this.list[i].hash == hash){
				this.list[i].hash = urlInfo.hash;
				this.list[i].longUrl = urlInfo.longUrl;
				this.list[i].short_url = urlInfo.short_url;
				break;
			}
		}
		this.notifyObservers("recentListChanged", this.list);
	},
	
	"store": function()
	{
		localStorage.setItem("recentUrls",JSON.stringify(this.list));
	},
	
	"restore": function()
	{
		var list = localStorage.getItem("recentUrls");
		if(list){
			this.list = JSON.parse(list);
		}else{
			this.list = [];
		}
		
		this.notifyObservers("recentListChanged", this.list);
	},
	
	"add": function(ulrData)
	{
		if(this.list.length >= 10){
			this.remove(this.list[0].hash);
		}
		
		this.list.push(ulrData);
		this.store();
		this.notifyObservers("recentListAdd", ulrData);
	},
	
	"remove": function(hash)
	{
		this.notifyObservers("recentListRemove", hash);
		for(var i=0;i<this.list.length;i++)
		{
			if(this.list[i].hash == hash){
				this.list.splice(i,1);
			}
		}
		this.store();
	},
	
	"massRemove": function(hashes)
	{
		for(var i=0;i<this.list.length;i++)
		{
			var index = hashes.indexOf(this.list[i].hash);
			if(index != -1){
				this.notifyObservers("recentListRemove", hashes[index]);
				this.list.splice(i,1);
			}
		}
		this.store();
	},
	
	"getShort": function(hash)
	{
		for(var i=0;i<this.list.length;i++)
		{
			if(this.list[i].hash == hash){
				return(this.list[i].short_url);
			}
		}
		return null;
	}
});



var UrlsList = $.extend({}, KBObservable.clone(),{
	firstTimeReloadDone: false,
	urlsCache:null,
	
	store: function(){
		localStorage.setItem("urlsCache",JSON.stringify(this.urlsCache));
		return this;
	},

	restore: function(){
		var urlsCache = localStorage.getItem("urlsCache");
		if(urlsCache){
			this.urlsCache = JSON.parse(urlsCache);
		}else{
			this.urlsCache = null;
		}
		
		this.notifyObservers("urlsListChanged", this.urlsCache);
		
		return this;
	},
	
	reload: function(params, force){
		if(force == true || this.firstTimeReloadDone == false){
			var self = this;
			var sortBy = $("#manage-search-form option:selected").val();
			var search = $("#manage-search-form input[name=search]").val();
			var localSortByNewClicks = false;
			
			if(sortBy == 'new'){ // special case - local sorting
				localSortByNewClicks = true;
				sortBy = 'created';
			}
			
			TinyCC.list_urls({
				"sortBy":sortBy,
				"search":search,
				"successCallback": function(response){
					for(var i=0;i<response.results.urls.length;i++){
						var oldUrl = self.getUrl(response.results.urls[i].hash);
						if(oldUrl){
							if(self.firstTimeReloadDone == false){
								response.results.urls[i].new_clicks = response.results.urls[i].total_clicks - oldUrl.total_clicks;
							}else{
								response.results.urls[i].new_clicks = (response.results.urls[i].total_clicks - oldUrl.total_clicks) + oldUrl.new_clicks;
							}
						}else{
							response.results.urls[i].new_clicks = 0;//response.results.urls[i].total_clicks;
						}
					}
					
					if(localSortByNewClicks){
						response.results.urls.sort(function(a,b){
							var result = b.new_clicks - a.new_clicks;
							if(result == 0){
								result = b.total_clicks - a.total_clicks;
							}
							return result;
						});
					}
					
					self.urlsCache = response.results;
					self.store().restore();
					self.firstTimeReloadDone = true;
					if(params.success)params.success();
				},
				"errorCallback": function(message){
					self.restore();
					self.firstTimeReloadDone = true;
					if(params.error)params.error(message);
				}
			});
		}else{
			if(params.success)params.success();
		}
		return this;
	},
	
	getUrl: function(hash, withDetails)
	{
		var urlInfo;
		
		if(!this.urlsCache
			|| !this.urlsCache.urls 
			|| !(this.urlsCache.urls instanceof Array)){
			return null;
		}
		
		for(var i=0;i<this.urlsCache.urls.length;i++)
		{
			if(this.urlsCache.urls[i].hash == hash){
				urlInfo = this.urlsCache.urls[i];
				break;
			}
		}
		
		if(urlInfo && withDetails === true){
			if(!urlInfo.details){
				urlInfo.details = TinyCC.getUrlDetails({"hash":hash,"async":false,
					"errorCallback":function(a){ MessageBox.error("Network error. Can't update referers!"); }
				});
				//console.log(this.urlsCache);
				urlInfo.details.custom_domain = this.urlsCache.custom_domain;
			}
		}
		
		return urlInfo;
	},
	
	findAndProcess: function(hash, processCallback)
	{
		for(var i=0;i<this.urlsCache.urls.length;i++)
		{
			if(this.urlsCache.urls[i].hash == hash){
				return processCallback(this.urlsCache.urls[i], i);
			}
		}
	},
	
	resetStats: function(hash)
	{
		this.findAndProcess(hash, function(urlInfo){
			TinyCC.resetStats({
				"hash":hash, 
				"successCallback":function(a){
					urlInfo.total_clicks = 0;
					urlInfo.unique_visits = 0;
					urlInfo.referrers = [];
				},
				"errorCallback": function(message){
					MessageBox.show(message);
				}
			});
		});
	},
	
	"remove": function(hash)
	{
		var self = this;
		this.findAndProcess(hash, function(urlInfo, index){
			TinyCC.remove({
				"hash":hash,
				"successCallback":function(a){
					self.urlsCache.urls.splice(index,1);
					self.urlsCache.count--;
					self.store().restore();
					RecentList.remove(hash);
				},
				"errorCallback": function(message){
					MessageBox.show(message);
				}
			});
		});
	},	
	
	massRemove: function(hashes)
	{
		var self = this;
		TinyCC.massRemove({
				"hashes":hashes,
				"successCallback":function(a){
						RecentList.massRemove(hashes);
						self.reload({"errorCallback": function(message){
							MessageBox.show(message);
						}},true);
				},
				"errorCallback": function(message){
					MessageBox.show(message);
				}
		});
	},
	
	"getShort": function(hash)
	{
		for(var i=0;i<this.urlsCache.urls.length;i++)
		{
			if(this.urlsCache.urls[i].hash == hash){
				return(this.urlsCache.urls[i].short_url);
			}
		}
		return null;
	},
	
	setFavorite: function(hash, favorite)
	{
		var self = this;
		this.findAndProcess(hash, function(urlInfo, index){
			TinyCC.setFavorite({
				"hash":hash,
				"favorite":favorite,
				"successCallback":function(a){
					self.urlsCache.urls[index].favorite = favorite;
					self.notifyObservers("urlsListChanged", self.urlsCache);
				},
				"errorCallback": function(message){
					MessageBox.show(message);
				}
			});
		});		
	},
	
	setQRMark: function(hash, isQR)
	{
		var self = this;
		this.findAndProcess(hash, function(urlInfo, index){
			TinyCC.setQRMark({
				"hash":hash,
				"is_qrcode":isQR,
				"successCallback":function(a){
					self.urlsCache.urls[index].is_qrcode = isQR;
					self.notifyObservers("urlsListChanged", self.urlsCache);
				},
				"errorCallback": function(message){
					MessageBox.show(message);
				}
			});
		});		
	},
	
	setEmailReport: function(hash, status)
	{
		var self = this;
		this.findAndProcess(hash, function(urlInfo, index){
			TinyCC.setEmailReport({
				"hash":hash,
				"email_stats":status,
				"successCallback":function(a){
					self.urlsCache.urls[index].email_stats = status;
					self.notifyObservers("urlsListChanged", self.urlsCache);
				},
				"errorCallback": function(message){
					MessageBox.show(message);
				}
			});
		});		
	},
	
	editUrl: function(params)
	{
		var self = this;
		this.findAndProcess(params.hash, function(urlInfo, index){
			TinyCC.editUrl({
				"hash":params.hash,
				"property":params.property,
				"value":params.value,
				"successCallback":function(a){
					if(params.property == 'customHash'){
						params.property = 'hash';
						self.urlsCache.urls[index]['short_url'] = 'http://'+self.urlsCache.custom_domain + '/'+params.value;
					}
					self.urlsCache.urls[index][params.property] = params.value;
					if(params.successCallback)params.successCallback(a);
					self.notifyObservers("urlsListChanged", self.urlsCache);
					RecentList.update(params.hash, self.urlsCache.urls[index]);
				},
				"errorCallback": function(message){
					MessageBox.show(message);
				}
			});
		});		
	},
	
/**
 * Reset new clicks conters when device resumed from background mode
 */
	resetNewClicks: function()
	{
		var self = this;
		for(var i=0;i<self.urlsCache.urls.length;i++){
			self.urlsCache.urls[i].new_clicks = 0;
		}
		self.store().restore();
	}

});

