/**
 * Tiny.cc REST API library
 * 
 */

var TinyCC = {
	options:{
		apiKey:"af8ef3a7-e3d0-4656-8e44-26a989829d0c", // default API key for mobile APP
		tinyServer:"https://d.tiny.cc",
		userName:"app",
		anonimus: true,
		custom_domain:null,
		allowedForAnonimus:[
			'shorten',
			'get_api_key'
		]
	},
	
	buildRequestUri: function(params, arrayParams)
	{
		var result = this.options.tinyServer+"/?c=rest_api";
		result+="&version=2.0.3&format=json";
		result+="&login="+this.options.userName+"&apiKey="+this.options.apiKey;
		for(var key in params){
			if(params.hasOwnProperty(key)){
				result+="&"+key+"="+encodeURIComponent(params[key]);
			}
		}
		for(var key in arrayParams){
			if(arrayParams.hasOwnProperty(key)){
				for(var i=0;i<arrayParams[key].length;i++){
					result+="&"+key+"[]="+encodeURIComponent(arrayParams[key][i]);
				}
			}
		}

		return result;
	},
	
	restApiCall: function(uri, params)
	{
		var result, async = true;
		if(params.async == false){
			async = false;
		}
		
		if(this.options.anonimus &&
			this.options.allowedForAnonimus[params.m]){
				throw new Error("This API method is allowed for registred users only!");
		}
		
		try{
			
			//$("#network-activity-indicator").show();
			
			$.ajax({
				"async": async,
				"url": uri,
				type: "post",
				dataType: "json",
				timeout: 60000, // 60s
				success: function(a){
					$("#network-activity-indicator").hide();
					if(a.errorCode && a.errorCode==0){
						if(async){
							params.successCallback(a);
						}else{
							result = a.results;
						}
					}else{
						if(params.errorCallback)params.errorCallback(a.errorMessage);
					}
				},
				error: function(){ 
					//$("#network-activity-indicator").hide();
					params.errorCallback("Network error. Try again later."); 
				} //function(a){ console.log(a); },
			});
		}catch(message){
			if(params.errorCallback)params.errorCallback(message);
			//$("#network-activity-indicator").hide();
		}
		
		return result;
	},
	
	"getApiKey": function(params)
	{
		this.restApiCall(this.buildRequestUri({"m":"get_api_key","password":params.password,"login":params.username}),params);
	},
	
	
	"shorten": function(params)
	{
		this.restApiCall(this.buildRequestUri({
				"m":"shorten",
				"longUrl":params.url,
				"shortUrl":params.customHash,
				"needStats":params.needStats
			}),params);
	},
	
	"list_urls": function(params)
	{
		this.restApiCall(this.buildRequestUri({"m":"list_urls","sortBy":params.sortBy,"search":params.search}),params);
	},
	
	getUrlDetails: function(params)
	{
		return this.restApiCall(this.buildRequestUri({"m":"get_url_details","hash":params.hash}),params);
	},
	
	"qrShow": function(hash)
	{
		return this.options.tinyServer+"/"+this.options.custom_domain+"/"+hash+"/qr/image/L/200";
	},

	"qrDownload": function(hash)
	{
		return this.options.tinyServer+"/"+this.options.custom_domain+"/"+hash+"/qr/download/L";
	},
	
	resetStats: function(params)
	{
		this.restApiCall(this.buildRequestUri({"m":"reset_stats","hash":params.hash}),params);
	},
	
	"remove": function(params)
	{
		this.restApiCall(this.buildRequestUri({"m":"delete","hash":params.hash}),params);
	},
	
	massRemove: function(params)
	{
		this.restApiCall(this.buildRequestUri({"m":"delete_batch"},{"hashes":params.hashes}),params);
	},
	
	setFavorite: function(params)
	{
		this.restApiCall(this.buildRequestUri({
				"m":"complex_edit",
				"hash":params.hash,
				"favorite":params.favorite
			}),params);
	},
	
	setQRMark: function(params)
	{
		this.restApiCall(this.buildRequestUri({
				"m":"complex_edit",
				"hash":params.hash,
				"isQrcode":params.is_qrcode
			}),params);
	},	
	
	editUrl: function(params)
	{
		var request = {"m":"complex_edit","hash":params.hash};
		request[params.property] = params.value;
		
		this.restApiCall(this.buildRequestUri(request),params);
	},		
	
	setEmailReport: function(params)
	{
		this.restApiCall(this.buildRequestUri({
				"m":"complex_edit",
				"hash":params.hash,
				"email_stats":params.email_stats
			}),params);
	},		
	
	getPrivateStatsUrl: function(hash)
	{
		return this.options.tinyServer+"/stats/"+hash+"/"+this.options.apiKey;
	}
};
