
var MessageBox = {
	"show": function(text, tiemout)
	{
		tiemout = tiemout ? tiemout : 3000;
		$("#message-box").finish().text(text).fadeIn().delay(tiemout).fadeOut();
	},
	
	"error": function(text, tiemout)
	{
		tiemout = tiemout ? tiemout : 3000;
		$("#message-box").finish().text(text)
			.prepend('<i class="fa fa-exclamation-triangle"></i> ')
			.fadeIn().delay(tiemout).fadeOut();
	},
};

var TemplateEngine = {
	"cache":{},
	monthLabels: ["January","February","March","April","May","June","July","August","September","October","November","December"],
	
	"init":function()
	{
		var self= this;
		//Handlebars.logger.log = function(a){ console.log(a); };
		
		Handlebars.registerHelper('prettyDate', function(dateString)
		{
			if (dateString && (dateString.length > 1))
			{
				var dateParts = dateString.match(/(\d{4})-(\d{2})-(\d{2})\s+(\d{2}):(\d{2}):(\d{2})/);
				
				if (dateParts.length >= 4)
				{
					dateParts[2] -= 1; // get correct month
					var date = new Date(Date.UTC.apply(this, dateParts.slice(1)));
					
					if (!isNaN(date.getTime()))
					{
						return new String(self.monthLabels[date.getMonth()]+" "+date.getDate()+", "+date.getFullYear()); //+" "+date.toLocaleTimeString();
					}
				}
			}

			// Try old way of reading date
			var date = new Date(dateString);

			if (!isNaN(date.getTime()))
			{
				return new String(self.monthLabels[date.getMonth()]+" "+date.getDate()+", "+date.getFullYear()); //+" "+date.toLocaleTimeString();
			}

			return 'N/A';
		});

		Handlebars.registerHelper('removeHTTP', function(dateString)
		{
			return dateString.replace(/^http:\/\//i,"");
		});

	},
	
	
	"use": function(templateId, arguments)
	{
		if(!this.cache[templateId]){
			var source = $("#"+templateId).html();
			var template = Handlebars.compile(source);
			this.cache[templateId] = template;
		}
		return this.cache[templateId](arguments);
	}
};


var TinyLayer = {
	confirm: function(message, yesCallback, noCallback)
	{
		// Cordova plugin support
		if(navigator.notification && navigator.notification.confirm){
			navigator.notification.confirm(message, function(buttonPressed){
				if(buttonPressed == 1){ // Ok button
					yesCallback();
					return;
				}

				if(buttonPressed == 2){ // Cancel button
					noCallback();
					return;
				}
			});
			return;
		}
		
		// Browser dialog fallback
		var result = window.confirm(message);
		if(result && yesCallback){
			yesCallback();
			return;
		}
		
		if(!result && noCallback){
			noCallback();
			return;
		}
	},
	
	share: function(shortUrl)
	{
		//MessageBox.show("Sharing "+shortUrl+". Not implemented yet!");
		
		try{
			navigator.share(shortUrl);
		}catch(e){
			MessageBox.error(e);
		}
	},
	
	copy: function(shortUrl)
	{
		try{
		    cordova.plugins.clipboard.copy(shortUrl);
		    MessageBox.show(shortUrl+" copied");
		}catch(e){
			MessageBox.error(e);
		}
	},
	
	readFromClipboard: function(callback)
	{
	//	MessageBox.show("Reading clipboard. Not implemented yet!");
		try{
			cordova.plugins.clipboard.paste(
                                        function (text)
                                        {
											callback(text);
                                        });
		}catch(e){
			MessageBox.error(e);
		}
	},
	
	download: function(srcFile, destFile)
	{
		if(/(android)/i.test(navigator.userAgent)){

			store = cordova.file.externalRootDirectory;

			var downloadProcedure = function(){
				var ft = new FileTransfer();
				ft.download(
					encodeURI(srcFile),
					(store + destFile),
					function(entry) {
						MessageBox.show("Download complete: " + entry.toURL());
					},
					function(error) {
						MessageBox.error("Error code: "+error.code);
					},
					true
				);
			}

			window.resolveLocalFileSystemURL(store + destFile, function(){}, downloadProcedure);

		}else{
			MessageBox.error("This function is now supported on iOS");
		}
	},
	
	popupBrowser: function(url)
	{
		window.open(url, '_blank', 'location=yes');
	}
};

var BusyIndicator = {
	show: function()
	{
		$("#busy-indicator").show();
		$("#refresh-urls-button").addClass("fa-spin");
	},
	
	hide: function()
	{
		$("#busy-indicator").hide();
		$("#refresh-urls-button").removeClass("fa-spin");
	}
	
};


var AdBanner = {
	init: function()
	{
		var admob_ios_key = 'ca-app-pub-1267983004660048/4813312145';
		var admob_android_key = 'ca-app-pub-1267983004660048/4813312145';
		var adId = (/(android)/i.test(navigator.userAgent)) ? admob_android_key : admob_ios_key;

		if(admob ) {
			
			admob.initAdmob(adId,adId);
			
			admob.showBanner(admob.BannerSize.BANNER,admob.Position.BOTTOM_APP);//show banner at the bottom of app 

			// Show Interstitial Ads
			admob.cacheInterstitial();
			$( ":mobile-pagecontainer" ).on( "pagecontainershow", function( event, ui ) {
				admob.isInterstitialReady(function(isReady){
					if(isReady){
						admob.showInterstitial();
					}
				});
			});

		
		} else {
			MessageBox.error('AdMob plugin not available/ready.');
		}



/*
 * Use of old com.rjfun.cordova.plugin.admob 2.1.4 "AdMob" plugin
 * 
		if( window.plugins && window.plugins.AdMob ) {
			var am = window.plugins.AdMob;

			am.createBannerView( 
				{
				'publisherId': adId,
				'adSize': am.AD_SIZE.BANNER,
				'bannerAtTop': false, // set to true, to make banner at top
				'overlap': false,  // set to true, to allow banner view overlap web content instead of push up/down
				'offsetTopBar': false // set to true, to avoid ios 7 status bar overlap
				}, 
				function() {
					am.requestAd( { 'isTesting':false }, // set to false, for production purpose 
						function(){}, 
						function(){ MessageBox.error('failed to request ad'); }
						);
				}, 
				function(){ MessageBox.error('failed to create banner view'); }
			);
		
		} else {
			MessageBox.error('AdMob plugin not available/ready.');
		}
		*/
	},
	
	initInterstitils: function()
	{
		//$( ":mobile-pagecontainer" ).on( "pagecontainershow", function( event, ui ) {
			//var activePage = $( "body" ).pagecontainer( "getActivePage" ).attr("id");
			//if(activePage == "manage"){
				//MessageBox.show('Interstitial here');
			//}
		//});
	}
}
